#
# DruMa - Teach an AitCS new moves.
#
# Stuart Cassoff
#
# Summer 2018
# Version 0.1
#
# Summer 2019
# Spring 2020
# Version 0.2
#

package require Tcl 8.6

namespace eval ::druma {

proc LICENSE {} { return [dict create type ISC text [string range {

Copyright (c) 2018,2020 Stuart Cassoff <stwo@users.sourceforge.net>

Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

} 2 end-2]] }; #esnecil

proc ABOUT {} { return [dict create name druma title DruMa descr "Teach an AitCS new moves" author {Stuart Cassoff} when {Spring 2020} version [package provide druma]] }


package require aitcs


oo::class create DruMa_Basic {
	variable name
	variable aitcs
constructor {aname {createAitCS {yes}}} {
	if {[set name [string trim $aname]] eq ""} { return -code error "name cannot be an empty string" }
	if {$createAitCS} { ::aitcs::aitcs new $name [uplevel 1 {namespace current}] [namespace current] }
	set aitcs [uplevel 1 $name&&]
	oo::objdefine [self] method destroyAitCS {} [interp alias {} $aitcs---]
	interp alias {} $aitcs--- {} [self] destroy
	interp alias {} $aitcs:   {} [self]
}
destructor {
	if {![info exists aitcs]} { return }
	interp alias {} $aitcs: {}
	my destroyAitCS
}
method unknown {{method {}} args} { return [expr {$method eq {} ? [self] : [next $method {*}$args]}] }
method name  {} { return $name  }
method aitcs {} { return $aitcs }
}; #oo::class create DruMa_Basic


# Needs the 'tu' extension
oo::class create DruMa {
	superclass DruMa_Basic
	# Superclass vars
	variable name
	variable aitcs
	# This class vars
	variable fields
	variable boolFields
	variable listFile
constructor {aname args} {
	package require tu
	next $aname {*}$args
	set fields [list]
	set boolFields [list]
	set listFile ""
	set DruMaConfigVars [list fields boolFields listFile]
	dict for {a v} $args {
		if {[set a [string range $a 1 end]] ni $DruMaConfigVars} { continue }
		set $a $v
	}
}
method fields args {
	if {[llength $args] == 1} { set fields [lindex $args 0] }
	return $fields
}
method boolFields args {
	if {[llength $args] == 1} { set boolFields [lindex $args 0] }
	return $boolFields
}
method listFile args {
	if {[llength $args] == 1} { set listFile [lindex $args 0] }
	return $listFile
}
method formatBool {bool} { return [expr {$bool ? "yes" : "no"}] }
method scanBool   {bool} { return [expr {[catch {expr {!!$bool}} b] ? 0 : $b}] }
method validateBools {} {
	foreach fld $boolFields {
		$aitcs $fld [my scanBool [$aitcs $fld]]
	}
}
method validate {} {
	my validateBools
}
method load {args} {
	if {[llength $args] == 1} { set listFile [lindex $args 0] }
	set fn $listFile
	if {!([file exists $fn] && [lassign [::tu::file::loadListFile $fn] fn data])} { return 0 }
	foreach l $data { lassign $l fld val; if {$fld in $fields} { $aitcs $fld $val } }
	my validate
	return 1
}
method save {args} {
	if {[llength $args] == 1} { set listFile [lindex $args 0] }
	set fn $listFile
	my validate
	return [lassign [::tu::saveFile $listFile [my pretty]] fn]
}
method pretty {} {
	set cl [list]
	set flds [lsort $fields]
	set width 0
	foreach fld $flds {
		set val [$aitcs $fld]
		if {$fld in $boolFields} {
			set val [my formatBool $val]
		}
		lappend vals [list $val]
		set width [::tcl::mathfunc::max $width [string length $fld]]
	}
	incr width
	foreach fld $flds val $vals { lappend cl [format "%-${width}s %s" [list $fld] $val] }
	lappend cl {}; # To get that last newline
	return [join $cl \n]
}
}; #oo::class create DruMa

namespace export DruMa

package provide druma 0.2

}; #namespace eval ::druma

# EOF
